@@ -1,5 +1,6 @@
 from itertools import chain, zip_longest
 from operator import methodcaller
+
 import regex as re
 
 from dateparser.utils import normalize_unicode
@@ -7,15 +8,42 @@ from dateparser.utils import normalize_unicode
 PARSER_HARDCODED_TOKENS = [":", ".", " ", "-", "/"]
 PARSER_KNOWN_TOKENS = ["am", "pm", "UTC", "GMT", "Z"]
 ALWAYS_KEEP_TOKENS = ["+"] + PARSER_HARDCODED_TOKENS
-KNOWN_WORD_TOKENS = ['monday', 'tuesday', 'wednesday', 'thursday', 'friday',
-                     'saturday', 'sunday', 'january', 'february', 'march',
-                     'april', 'may', 'june', 'july', 'august', 'september',
-                     'october', 'november', 'december', 'decade', 'year',
-                     'month', 'week', 'day', 'hour', 'minute', 'second', 'ago',
-                     'in', 'am', 'pm']
+KNOWN_WORD_TOKENS = [
+    "monday",
+    "tuesday",
+    "wednesday",
+    "thursday",
+    "friday",
+    "saturday",
+    "sunday",
+    "january",
+    "february",
+    "march",
+    "april",
+    "may",
+    "june",
+    "july",
+    "august",
+    "september",
+    "october",
+    "november",
+    "december",
+    "decade",
+    "year",
+    "month",
+    "week",
+    "day",
+    "hour",
+    "minute",
+    "second",
+    "ago",
+    "in",
+    "am",
+    "pm",
+]
 
-PARENTHESES_PATTERN = re.compile(r'[\(\)]')
-NUMERAL_PATTERN = re.compile(r'(\d+)')
+PARENTHESES_PATTERN = re.compile(r"[\(\)]")
+NUMERAL_PATTERN = re.compile(r"(\d+)")
 KEEP_TOKEN_PATTERN = re.compile(r"^.*[^\W_].*$", flags=re.U)
 
 
@@ -49,29 +77,31 @@ class Dictionary:
         self._settings = settings
         self.info = locale_info
 
-        if 'skip' in locale_info:
-            skip = map(methodcaller('lower'), locale_info['skip'])
+        if "skip" in locale_info:
+            skip = map(methodcaller("lower"), locale_info["skip"])
             dictionary.update(zip_longest(skip, [], fillvalue=None))
-        if 'pertain' in locale_info:
-            pertain = map(methodcaller('lower'), locale_info['pertain'])
+        if "pertain" in locale_info:
+            pertain = map(methodcaller("lower"), locale_info["pertain"])
             dictionary.update(zip_longest(pertain, [], fillvalue=None))
         for word in KNOWN_WORD_TOKENS:
             if word in locale_info:
-                translations = map(methodcaller('lower'), locale_info[word])
+                translations = map(methodcaller("lower"), locale_info[word])
                 dictionary.update(zip_longest(translations, [], fillvalue=word))
         dictionary.update(zip_longest(ALWAYS_KEEP_TOKENS, ALWAYS_KEEP_TOKENS))
-        dictionary.update(zip_longest(map(methodcaller('lower'),
-                                          PARSER_KNOWN_TOKENS),
-                                      PARSER_KNOWN_TOKENS))
+        dictionary.update(
+            zip_longest(
+                map(methodcaller("lower"), PARSER_KNOWN_TOKENS), PARSER_KNOWN_TOKENS
+            )
+        )
 
-        relative_type = locale_info.get('relative-type', {})
+        relative_type = locale_info.get("relative-type", {})
         for key, value in relative_type.items():
-            relative_translations = map(methodcaller('lower'), value)
+            relative_translations = map(methodcaller("lower"), value)
             dictionary.update(zip_longest(relative_translations, [], fillvalue=key))
 
         self._dictionary = dictionary
 
-        no_word_spacing = locale_info.get('no_word_spacing', 'False')
+        no_word_spacing = locale_info.get("no_word_spacing", "False")
         self._no_word_spacing = bool(eval(no_word_spacing))
 
         relative_type_regex = locale_info.get("relative-type-regex", {})
@@ -143,6 +173,14 @@ class Dictionary:
 
         return list(filter(bool, chain.from_iterable(tokens)))
 
+    def _add_to_cache(self, value, cache):
+        cache.setdefault(self._settings.registry_key, {})[self.info["name"]] = value
+        if (
+            self._settings.CACHE_SIZE_LIMIT
+            and len(cache) > self._settings.CACHE_SIZE_LIMIT
+        ):
+            cache.pop(list(cache.keys())[0])
+
     def _split_by_known_words(self, string, keep_formatting):
         if not string:
             return string
@@ -150,8 +188,11 @@ class Dictionary:
         regex = self._get_split_regex_cache()
         match = regex.match(string)
         if not match:
-            return (self._split_by_numerals(string, keep_formatting)
-                    if self._should_capture(string, keep_formatting) else [])
+            return (
+                self._split_by_numerals(string, keep_formatting)
+                if self._should_capture(string, keep_formatting)
+                else []
+            )
 
         unparsed, known, unknown = match.groups()
         splitted = [known] if self._should_capture(known, keep_formatting) else []
@@ -163,87 +204,125 @@ class Dictionary:
         return splitted
 
     def _split_by_numerals(self, string, keep_formatting):
-        return [token for token in NUMERAL_PATTERN.split(string)
-                if self._should_capture(token, keep_formatting)]
+        return [
+            token
+            for token in NUMERAL_PATTERN.split(string)
+            if self._should_capture(token, keep_formatting)
+        ]
 
     def _should_capture(self, token, keep_formatting):
-        return keep_formatting or token in ALWAYS_KEEP_TOKENS or KEEP_TOKEN_PATTERN.match(token)
+        return (
+            keep_formatting
+            or token in ALWAYS_KEEP_TOKENS
+            or KEEP_TOKEN_PATTERN.match(token)
+        )
 
     def _get_sorted_words_from_cache(self):
         if (
             self._settings.registry_key not in self._sorted_words_cache
-            or self.info['name'] not in self._sorted_words_cache[self._settings.registry_key]
+            or self.info["name"]
+            not in self._sorted_words_cache[self._settings.registry_key]
         ):
-            self._sorted_words_cache.setdefault(
-                self._settings.registry_key, {})[self.info['name']] = \
-                sorted([key for key in self], key=len, reverse=True)
-        return self._sorted_words_cache[self._settings.registry_key][self.info['name']]
+            self._add_to_cache(
+                cache=self._sorted_words_cache,
+                value=sorted([key for key in self], key=len, reverse=True),
+            )
+        return self._sorted_words_cache[self._settings.registry_key][self.info["name"]]
 
     def _get_split_regex_cache(self):
         if (
             self._settings.registry_key not in self._split_regex_cache
-            or self.info['name'] not in self._split_regex_cache[self._settings.registry_key]
+            or self.info["name"]
+            not in self._split_regex_cache[self._settings.registry_key]
         ):
             self._construct_split_regex()
-        return self._split_regex_cache[self._settings.registry_key][self.info['name']]
+        return self._split_regex_cache[self._settings.registry_key][self.info["name"]]
 
     def _construct_split_regex(self):
-        known_words_group = "|".join(map(re.escape, self._get_sorted_words_from_cache()))
+        known_words_group = "|".join(
+            map(re.escape, self._get_sorted_words_from_cache())
+        )
         if self._no_word_spacing:
             regex = r"^(.*?)({})(.*)$".format(known_words_group)
         else:
-            regex = r"^(.*?(?:\A|\W|_|\d))({})((?:\Z|\W|_|\d).*)$".format(known_words_group)
-        self._split_regex_cache.setdefault(
-            self._settings.registry_key, {})[self.info['name']] = \
-            re.compile(regex, re.UNICODE | re.IGNORECASE)
+            regex = r"^(.*?(?:\A|\W|_|\d))({})((?:\Z|\W|_|\d).*)$".format(
+                known_words_group
+            )
+        self._add_to_cache(
+            cache=self._split_regex_cache,
+            value=re.compile(regex, re.UNICODE | re.IGNORECASE),
+        )
 
     def _get_sorted_relative_strings_from_cache(self):
         if (
             self._settings.registry_key not in self._sorted_relative_strings_cache
-            or self.info['name'] not in self._sorted_relative_strings_cache[self._settings.registry_key]
+            or self.info["name"]
+            not in self._sorted_relative_strings_cache[self._settings.registry_key]
         ):
-            self._sorted_relative_strings_cache.setdefault(
-                self._settings.registry_key, {})[self.info['name']] = \
-                sorted([PARENTHESES_PATTERN.sub('', key) for key in
-                        self._relative_strings], key=len, reverse=True)
-        return self._sorted_relative_strings_cache[self._settings.registry_key][self.info['name']]
+            self._add_to_cache(
+                cache=self._sorted_relative_strings_cache,
+                value=sorted(
+                    [
+                        PARENTHESES_PATTERN.sub("", key)
+                        for key in self._relative_strings
+                    ],
+                    key=len,
+                    reverse=True,
+                ),
+            )
+        return self._sorted_relative_strings_cache[self._settings.registry_key][
+            self.info["name"]
+        ]
 
     def _get_split_relative_regex_cache(self):
         if (
             self._settings.registry_key not in self._split_relative_regex_cache
-            or self.info['name'] not in self._split_relative_regex_cache[self._settings.registry_key]
+            or self.info["name"]
+            not in self._split_relative_regex_cache[self._settings.registry_key]
         ):
             self._construct_split_relative_regex()
-        return self._split_relative_regex_cache[self._settings.registry_key][self.info['name']]
+        return self._split_relative_regex_cache[self._settings.registry_key][
+            self.info["name"]
+        ]
 
     def _construct_split_relative_regex(self):
-        known_relative_strings_group = "|".join(self._get_sorted_relative_strings_from_cache())
+        known_relative_strings_group = "|".join(
+            self._get_sorted_relative_strings_from_cache()
+        )
         if self._no_word_spacing:
             regex = "({})".format(known_relative_strings_group)
         else:
-            regex = "(?<=(?:\\A|\\W|_))({})(?=(?:\\Z|\\W|_))".format(known_relative_strings_group)
-        self._split_relative_regex_cache.setdefault(
-            self._settings.registry_key, {})[self.info['name']] = \
-            re.compile(regex, re.UNICODE | re.IGNORECASE)
+            regex = "(?<=(?:\\A|\\W|_))({})(?=(?:\\Z|\\W|_))".format(
+                known_relative_strings_group
+            )
+        self._add_to_cache(
+            cache=self._split_relative_regex_cache,
+            value=re.compile(regex, re.UNICODE | re.IGNORECASE),
+        )
 
     def _get_match_relative_regex_cache(self):
         if (
             self._settings.registry_key not in self._match_relative_regex_cache
-            or self.info['name'] not in self._match_relative_regex_cache[self._settings.registry_key]
+            or self.info["name"]
+            not in self._match_relative_regex_cache[self._settings.registry_key]
         ):
             self._construct_match_relative_regex()
-        return self._match_relative_regex_cache[self._settings.registry_key][self.info['name']]
+        return self._match_relative_regex_cache[self._settings.registry_key][
+            self.info["name"]
+        ]
 
     def _construct_match_relative_regex(self):
-        known_relative_strings_group = "|".join(self._get_sorted_relative_strings_from_cache())
+        known_relative_strings_group = "|".join(
+            self._get_sorted_relative_strings_from_cache()
+        )
         regex = "^({})$".format(known_relative_strings_group)
-        self._match_relative_regex_cache.setdefault(
-            self._settings.registry_key, {})[self.info['name']] = \
-            re.compile(regex, re.UNICODE | re.IGNORECASE)
+        self._add_to_cache(
+            cache=self._match_relative_regex_cache,
+            value=re.compile(regex, re.UNICODE | re.IGNORECASE),
+        )
 
 
 class NormalizedDictionary(Dictionary):
-
     def __init__(self, locale_info, settings=None):
         super().__init__(locale_info, settings)
         self._normalize()
@@ -259,7 +338,7 @@ class NormalizedDictionary(Dictionary):
                 new_dict[normalized] = value
         for key in conflicting_keys:
             normalized = normalize_unicode(key)
-            if key in (self.info.get('skip', []) + self.info.get('pertain', [])):
+            if key in (self.info.get("skip", []) + self.info.get("pertain", [])):
                 new_dict[normalized] = self._dictionary[key]
         self._dictionary = new_dict
         self._relative_strings = list(map(normalize_unicode, self._relative_strings))
